#!/home/mfleck/mfleck/schwa/demo/interp

;; University of Illinois/NCSA
;; Open Source License
;; 
;; Copyright © 2005, The Board of Trustees of the University of Illinois.
;; All rights reserved.
;; Developed by:
;; 
;;   Department of Computer Science
;;   University of Illinois, Urbana-Champaign
;;   http://www.cs.uiuc.edu
;; 
;; Permission is hereby granted, free of charge, to any person obtaining
;; a copy of this software and associated documentation files (the
;; "Software"), to deal with the Software without restriction, including
;; without limitation the rights to use, copy, modify, merge, publish,
;; distribute, sublicense, and/or sell copies of the Software, and to
;; permit persons to whom the Software is furnished to do so, subject to
;; the following conditions:
;; 
;;   * Redistributions  of  source  code  must retain the above copyright
;;     notice, this list of conditions and the following disclaimers.
;;   * Redistributions  in binary form must reproduce the above copyright
;;     notice, this  list of conditions and the following disclaimers in
;;     the documentation and/or other materials provided with the
;;     distribution.
;;   * Neither the names of the Department of Computer Science,
;;     University of Illinois, Urbana-Champaign, nor the names of its
;;     contributors may be used to endorse or promote products derived 
;;     from this Software without specific prior written permission.
;; 
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
;; EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
;; MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
;; IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR
;; ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
;; CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
;; WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.




;; Read a plaintext file and convert each word to
;;   its pronunciation in a dictionary of the same
;;   format as the CMU pronouncing dictionary.
;; Margaret M. Fleck, 2005

;; Debugging output has been left on, for illustration.
(debug_on)

;; Edit this line to read pronunciations from
;; a different dictionary file, e.g. the whole 
;; CMU pronouncing dictionary.
(set cmudict (read_cmudict "shortdict"))
	      
;; Try using gettysburg.txt as the input file
(set usage "Usage:  ./pronounce.scm infile > outfile")

(when (not (equal (length *args*) 1))
      (error "Wrong number of args \n %x\n" usage)) 
	
;; Display the pronunciation of the token.
;; Tokens with empty pronunciation (e.g. punctuation) are silently omitted.
;; Tokens not in dictionary are displayed as [??word??]
(set showword
     (lambda (word)
       (if (equal (item2type word) 'symbol)
	   (set word (list word)))
       (dolist (token word)
	       (set entry (hashget cmudict (token2lower token)))
	       (if (present entry)
		   (begin 
		     (set pronun (car (bundleget entry 'pronunciations)))
		     (if (> (length pronun) 0)          ;; e.g. token is punctuation
			 (format_infull "%s " pronun)))
		   (format_infull "[??%s??] " token)))))  ;; word is not in dictionary

(set showline
     (lambda (line)
       (dolist (word line)
	       (showword word))
       (format "\n")))

(io_to *stderr* (format "Converting input file\n"))
(set myfile (open_input_file (car *args*)))
(while (not (at_end (set lineoftokens (tokenize_next_line myfile))))
	(showline lineoftokens))

(close_file myfile)
