#!/home/mfleck/mfleck/schwa/demo/interp

;; University of Illinois/NCSA
;; Open Source License
;; 
;; Copyright © 2005, The Board of Trustees of the University of Illinois.
;; All rights reserved.
;; Developed by:
;; 
;;   Department of Computer Science
;;   University of Illinois, Urbana-Champaign
;;   http://www.cs.uiuc.edu
;; 
;; Permission is hereby granted, free of charge, to any person obtaining
;; a copy of this software and associated documentation files (the
;; "Software"), to deal with the Software without restriction, including
;; without limitation the rights to use, copy, modify, merge, publish,
;; distribute, sublicense, and/or sell copies of the Software, and to
;; permit persons to whom the Software is furnished to do so, subject to
;; the following conditions:
;; 
;;   * Redistributions  of  source  code  must retain the above copyright
;;     notice, this list of conditions and the following disclaimers.
;;   * Redistributions  in binary form must reproduce the above copyright
;;     notice, this  list of conditions and the following disclaimers in
;;     the documentation and/or other materials provided with the
;;     distribution.
;;   * Neither the names of the Department of Computer Science,
;;     University of Illinois, Urbana-Champaign, nor the names of its
;;     contributors may be used to endorse or promote products derived 
;;     from this Software without specific prior written permission.
;; 
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
;; EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
;; MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
;; IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR
;; ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
;; CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
;; WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.



;; Compute bigram frequencies for Mississippi State Switchboard transcriptions
;;    Margaret M. Fleck, 2005

;; These transcriptions can be obtained (free!) from
;;     http://www.cavs.msstate.edu/hse/ies/projects/switchboard/
;;
;; This code illustrates how to use the Switchboard transcription reader code,
;; how to walk a set of directories too long for a single ls command,
;; and also how to use an earray to store information about ngrams.
;;
;; Usage:  ./swbd_bigram.scm > outfile
;;
;; Output frequencies are listed in random order.   Try running the output through sort:
;;    sort -rg -k1,1 outfile > sortedfile

(debug_on)

(set directories (ls "/corpora/switchboard-icip/swb_ms98_transcriptions/[0-9]*/"))

;; When debugging, you may wish to walk only a subset of the corpus.  Try
;;     something like the following:
;;(set directories (list "/corpora/switchboard-icip/swb_ms98_transcriptions/20/"))


(set table (new_counttable))

;; The ls command refuses to list really long sets of files, so we enumerate them
;; directory-by-directory, using a series of smaller ls commands.

(dolist (directory directories)
	(io_to *stderr* (format "Entering directory %s\n" directory))
	(set files (ls (sformat "%s*/*trans*.text" directory)))
	(dolist (file files)
		(set text (swbd2plain (read_swbd_file file)))
		(count_bigrams text table)))

(print_bigrams table)
