#!/home/mfleck/mfleck/schwa/demo/interp

;; University of Illinois/NCSA
;; Open Source License
;; 
;; Copyright © 2005, The Board of Trustees of the University of Illinois.
;; All rights reserved.
;; Developed by:
;; 
;;   Department of Computer Science
;;   University of Illinois, Urbana-Champaign
;;   http://www.cs.uiuc.edu
;; 
;; Permission is hereby granted, free of charge, to any person obtaining
;; a copy of this software and associated documentation files (the
;; "Software"), to deal with the Software without restriction, including
;; without limitation the rights to use, copy, modify, merge, publish,
;; distribute, sublicense, and/or sell copies of the Software, and to
;; permit persons to whom the Software is furnished to do so, subject to
;; the following conditions:
;; 
;;   * Redistributions  of  source  code  must retain the above copyright
;;     notice, this list of conditions and the following disclaimers.
;;   * Redistributions  in binary form must reproduce the above copyright
;;     notice, this  list of conditions and the following disclaimers in
;;     the documentation and/or other materials provided with the
;;     distribution.
;;   * Neither the names of the Department of Computer Science,
;;     University of Illinois, Urbana-Champaign, nor the names of its
;;     contributors may be used to endorse or promote products derived 
;;     from this Software without specific prior written permission.
;; 
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
;; EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
;; MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
;; IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR
;; ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
;; CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
;; WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.




;; Extract and display the titles from a file of songs

;; Margaret Fleck, 2005

;; This script illustrates the use of open_datafile, which can read
;;    possibly-compressed input files.   It also illustrates reading
;;    an SGML file by meaningful chunks, rather than (a) one line or 
;;    token at a time or (b) whole file at once.   This is useful
;;    when processing corpora where many small documents are packed
;;    into a large file, e.g English Gigaword.


(when (not (equal (length *args*) 0))
      (error "Usage:  ./titles.scm \n"))



;; Extract and print title for this song
(set process_song
     (lambda (songtext)
       (let ((title (extract_sgml_block songtext 'title)))
	 (show_utterances *evalout* (sgml2txt title '())))))



(set process_file 
     (lambda (filename)
       (let ((file (open_datafile filename))  ;; use open_datafile because it's probably compressed
	     (i 0)
	     (type)
	     (textblock))
	 (while (present (set contents (read_sgml_block file 'song 0 '())))
		(process_song contents))
	 (close_datafile file))))
	 
(format "\nTITLES FROM REGULAR SGML FILE\n")
(process_file "songs.sgml")

(format "\nTITLES FROM COMPRESSED SGML FILE\n")
(process_file "songs.sgml.gz")

