/*
University of Illinois/NCSA
Open Source License

Copyright © 2005, The Board of Trustees of the University of Illinois.
All rights reserved.
Developed by:

  Department of Computer Science
  University of Illinois, Urbana-Champaign
  http://www.cs.uiuc.edu

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal with the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

  * Redistributions  of  source  code  must retain the above copyright
    notice, this list of conditions and the following disclaimers.
  * Redistributions  in binary form must reproduce the above copyright
    notice, this  list of conditions and the following disclaimers in
    the documentation and/or other materials provided with the
    distribution.
  * Neither the names of the Department of Computer Science,
    University of Illinois, Urbana-Champaign, nor the names of its
    contributors may be used to endorse or promote products derived 
    from this Software without specific prior written permission.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR
ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.

*/

/* The main external definitions file for schwa. */

#ifndef schwaeval
#define schwaeval

#include "lists.h"
#include "earray.h"
#include <signal.h>

/* The environment in which the interpreter is executing. */
struct schwa_env {
  Hashtable *globalbindings;     /* Global variable bindings. */
  Item localbindings;            /* Local variable bindings.   A list of pairs
				    (symbol, value).  This are consulted first
                                    before dynamic bindings or global environment.  */
  Item dynamicbindings;          /* Dynamic variable bindings.  These are consulted
				    after localbindings but before global environment. */
  int leavelive;                 /* Non-zero iff interactive shell should be 
				    invoked after executing shell script.   
				    Useful for debugging or to customize 
				    interactive shell.   Will not do the right
                                    thing if you have redirected stdin or 
				    stdout. */
  int errorrestart;              /* Should interpreter restart after error? */
  char *prompt;                  /* Prompt used to elicit input from user. */
  int force_echo;                /* Force shell script to echo command 
				    input/outputs */
  int breakisset;                /* breakisset, breakpoint, and breakvalue
				    help implement the block/return facility
				    for non-local jumps */
  jmp_buf *breakpoint;
  Item breakvalue;
  Item backtrace;               /* List of active function calls */
};

typedef struct schwa_env Schwa_env;

/* Printer will only recurse PRINDEPTH levels into structure  
   and will only print the first PRINLENGTH elements of a list.
   This prevents an infinite loop on circular structures.
   Linewidth is the target line length for pretty printing.
   Feel free to reset these variables. */
extern unsigned int PRINDEPTH;  
extern unsigned int PRINLENGTH;  
extern unsigned int LINEWIDTH;


/* Functions defined by eval.c */
void defun(char *name, void *function, int arity, Schwa_env *environment);
void defun_vararg(char *name, void *function, Schwa_env *environment);
void defun_sf(char *name, void *function, Schwa_env *environment);
void setsymbol(Item symbol, Item value, Schwa_env *environment);
void invoke_interpreter(int argc, char *argv[], Schwa_env *environment);
Item eval(Item expression, Schwa_env *environment);
Item evalsequence(Item exprs, Schwa_env *environment);
Item evalsymbol(Item symbol, Schwa_env *environment);
Item evalargs(Item exprs, Schwa_env *environment);
void evalfile(char *filename, Schwa_env *environment, int interactive);
Item apply(Item head, Item args, Schwa_env *environment);

extern const Item FALSE;
extern const Item TRUE;

Schwa_env *null_env();
Schwa_env *standard_env();

/* Typedefs for doing eval of functions stored as Items.   Do they
   need to be in the external interface or can they be moved into lists.c? */
typedef Item (*FUNC0)();
typedef Item (*FUNC1)(Item);
typedef Item (*FUNC2)(Item, Item);
typedef Item (*FUNC3)(Item, Item, Item);
typedef Item (*FUNC4)(Item, Item, Item, Item);
typedef Item (*FUNC5)(Item, Item, Item, Item, Item);
typedef Item (*FUNC6)(Item, Item, Item, Item, Item, Item);
typedef Item (*FUNC7)(Item, Item, Item, Item, Item, Item, Item);
typedef Item (*FUNC8)(Item, Item, Item, Item, Item, Item, Item, Item);

/* Typedef for special forms. */
typedef Item (*FUNCSF)(Item args, Schwa_env *environment);

#endif

