/*
University of Illinois/NCSA
Open Source License

Copyright © 2005, The Board of Trustees of the University of Illinois.
All rights reserved.
Developed by:

  Department of Computer Science
  University of Illinois, Urbana-Champaign
  http://www.cs.uiuc.edu

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal with the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

  * Redistributions  of  source  code  must retain the above copyright
    notice, this list of conditions and the following disclaimers.
  * Redistributions  in binary form must reproduce the above copyright
    notice, this  list of conditions and the following disclaimers in
    the documentation and/or other materials provided with the
    distribution.
  * Neither the names of the Department of Computer Science,
    University of Illinois, Urbana-Champaign, nor the names of its
    contributors may be used to endorse or promote products derived 
    from this Software without specific prior written permission.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR
ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.

*/


#ifndef schwasymbol
#define schwasymbol

#include "utils.h"

/* largest unsigned integer that will fit in 29 bits */
#define MAXCODE    (256*256*256*32 - 1)    

/* Linked lists of hash codes. */
struct codelist {
  unsigned long value;
  struct codelist *next;
};

typedef struct codelist Codelist;


/* A symbol table.  
   The forward table is an array mapping codes to strings.
   The hash table then maps strings to codes. */
struct symboltable {
  char ***forwardtable;          /* Table mapping codes to strings, 
				    subdivided into blocks. */
  unsigned long forwardsize;     /* Length of array of blocks. */
  unsigned long maxelement;      /* First free position in forward table. */
  Codelist ***hashtable;         /* Also subdivided into blocks. */
  unsigned long hashblocklen;    /* Length of array of blocks. */
  unsigned long hashtablesize;   /* Nominal length of hash table, 
				    used in hashing. */
  unsigned long firstcode;       /* Lowest position we can use in table.
				    Allows creator to reserve a few 
				    positions at start. */
  
};

typedef struct symboltable Symboltable;

Symboltable *new_symboltable (unsigned int numreserve);
long intern_string (char *string, Symboltable *table, int autoadd);
char *code2string (unsigned long code, Symboltable *table);

/* void print_occupancy(Symboltable *table, int groupsize); */
/* void print_forward_table(Symboltable *table); */

#endif
